library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.my320pkg.all;

entity char_controller is
   port(
     clk : in std_logic;
     
     
     switches : in std_logic_vector(7 downto 0);
     button0 : in std_logic;
     button1 : in std_logic;
     button2 : in std_logic;
     button3 : in std_logic;
     RGB : out std_logic_vector(2 downto 0);
     HS, VS : out std_logic;  
    
    segments : out std_logic_vector(6 downto 0);
		dp_out : out std_logic;
		an_sel: out std_logic_vector(3 downto 0)
     );
        
end char_controller;

architecture controller of char_controller is
    
    component default_screen is
      port(
        clk : in std_logic;
        en : in std_logic;
        we : in std_logic;
        addra : in std_logic_vector(14 downto 0);
        addrb : in std_logic_vector(14 downto 0);
        
        di : in std_logic_vector(3 downto 0);
        doa : out std_logic_vector(3 downto 0);
        dob : out std_logic_vector(3 downto 0)
       );
    end component;
        
    signal blank : std_logic;
    
    signal we1, we2 : std_logic := '0';
    signal reset : std_logic := '1';
     
     -- value going to be writen to mem
    signal write_data : std_logic_vector(3 downto 0);
	
     
     -- address of write and read
    signal  read_write_addr : unsigned(14 downto 0) := (others=>'0');
     
     -- value when reading and not writing
    signal  read_write_value, read_write_value1, read_write_value2 : std_logic_vector(3 downto 0);
     
     
     -- always reading to display pixel
    signal vga_addr : unsigned(19 downto 0);
    signal  read_addr : std_logic_vector(14 downto 0);     
    signal read_value, read_value1, read_value2 : std_logic_vector(3 downto 0);
    
     -- keeps track of address
    signal pixel_x, pixel_y : std_logic_vector(9 downto 0);
    
    -- registers that hold surrounding pixel values
    signal r1c1_reg, r1c2_reg, r1c3_reg,
	      r2c1_reg, r2c2_reg, r2c3_reg,
           r3c1_reg, r3c2_reg, r3c3_reg : std_logic_vector(6 downto 0) := (others=>'0');

    signal r1c1_next, r1c2_next, r1c3_next,
	      r2c1_next, r2c2_next, r2c3_next,
           r3c1_next, r3c2_next, r3c3_next : std_logic_vector(6 downto 0) := (others=>'0');

    -- state machine
    type state_type is (newline1, newline2, newline3, read1, read2, read3, write1, rand_init, nothing, idle);
    signal state_reg, state_next: state_type :=   newline2;

    signal write_pointer, write_pointer_next : unsigned(14 downto 0) := (others=>'0');
    
    signal sum : unsigned(3 downto 0);


    signal pixel_in : std_logic := '0';
    
    
    --Signals for Rnadmon_Init
    signal init_en, init_done : std_logic :='0';
    signal init_data : std_logic_vector(3 downto 0);
    signal init_addr : std_logic_vector(14 downto 0);
    
    signal delay_counter_reg, delay_counter_next : unsigned(25 downto 0) := (others=>'0');

    signal background_color : unsigned(2 downto 0):= (others => '0');
    signal change_colors : std_logic := '0';

    signal frame : std_logic := '1';
    
    signal speed, speed_reg, speed_next : unsigned(25 downto 0) := to_unsigned(100,26);
    
    signal temp1, temp2, temp3 : std_logic_vector(3 downto 0);
    
    signal x_place, x_place_reg : unsigned(8 downto 0) := (others=>'0');

begin
  
  process(clk)
  begin
      
    if(clk'event and clk = '1') then
      pixel_in <= not pixel_in;
  
      reset <='0';
  
      r1c1_reg <= r1c1_next;
      r1c2_reg <= r1c2_next;
      r1c3_reg <= r1c3_next;
      r2c1_reg <= r2c1_next;
      r2c2_reg <= r2c2_next;
      r2c3_reg <= r2c3_next;
      r3c1_reg <= r3c1_next;
      r3c2_reg <= r3c2_next;
      r3c3_reg <= r3c3_next;
      
      speed_reg <= speed_next;
      
      delay_counter_reg <= delay_counter_next;
      x_place <= x_place_reg;
      
      if(init_en='1') then
        state_reg <= rand_init;
      else 
        state_reg <= state_next;
      end if;
      
      if(change_colors='1') then
          background_color <= unsigned(switches(2 downto 0));
      end if;

	    write_pointer <= write_pointer_next;
      
      
    end if;
      
  end process;

  ----------------------------
  -- ADD MORE SIGNALS       --
  ----------------------------
  process(state_reg, r1c2_reg, r1c3_reg, r2c1_reg, r2c2_reg, r2c3_reg, r3c1_reg, r3c2_reg,
          r3c3_reg, read_write_value, write_pointer, sum, r1c1_reg, init_data, init_done,
          delay_counter_reg, x_place, speed_reg )
  begin
     --we <=  '0';
	   write_pointer_next <= write_pointer;
	   --read_write_addr <= (others=>'0');
		 write_data <= (others =>'0');
	   
	    r1c1_next <= r1c1_reg;
      r1c2_next <= r1c2_reg;
      r1c3_next <= r1c3_reg;
      r2c1_next <= r2c1_reg;
      r2c2_next <= r2c2_reg;
      r2c3_next <= r2c3_reg;
      r3c1_next <= r3c1_reg;
      r3c2_next <= r3c2_reg;
      r3c3_next <= r3c3_reg;
      delay_counter_next <= delay_counter_reg;
      x_place_reg <= x_place;
     
     case state_reg is
	     
         when newline1 =>
	         r1c2_next <= "000" & read_write_value;
	         --read_write_addr <= write_pointer;
		 	r1c1_next <= (others=>'0');
          r2c1_next <= (others=>'0');
          r3c1_next <= (others=>'0');
          r1c2_next <= (others=>'0');
                    r3c2_next <= (others=>'0');
					r1c3_next <= (others=>'0');
					r2c3_next <= (others=>'0');
					r3c3_next <= (others=>'0');

	         state_next <= newline2;
	     
        when newline2 =>
	         r2c2_next <= "000" & read_write_value;
	         --read_write_addr <= write_pointer;

	         state_next <= newline3;

	      when newline3 =>
		       r3c2_next <= "000" & read_write_value;
		       --read_write_addr <= write_pointer + 120;

		       state_next <= read1;
		       
		    when idle =>
		      if(delay_counter_reg = speed_reg) then
		        delay_counter_next <= (others=>'0');
			if(x_place=159) then
				x_place_reg <= (others=>'0');
				state_next <= newline1;
			else x_place_reg <= x_place +1;
				state_next <= read1;
			end if;
		      else
		        delay_counter_next <= delay_counter_reg + 1;
		        state_next <= idle;
		      end if;

        when read1 =>
          -- how about top right corner?
		      --if((write_pointer - 120) < 0 or unsigned(pixel_x) = 159) then
		      if((write_pointer) < 160 or x_place >=158) then
		         r1c3_next <= (others=>'0');
		      else
            r1c3_next <= "000" & read_write_value;
		        --read_write_addr <= write_pointer - 119;
		      end if;
		      r1c1_next <= r1c2_reg;
		      r1c2_next <= r1c3_reg;

	        state_next <= read2;

        when read2 =>
		      if( x_place >=158) then
		        r2c3_next <= (others=>'0');
		      else
		             r2c3_next <= "000" & read_write_value;
             --read_write_addr <= write_pointer + 1;
		      end if;
          
          r2c1_next <= r2c2_reg;
		      r2c2_next <= r2c3_reg;
		      
	        state_next <= read3;

	      when read3 =>
		      if((write_pointer) > 19040 or x_place >=158) then
		        r3c3_next <= (others=>'0');
		      else
            r3c3_next <= "000" & read_write_value;
            --read_write_addr <= write_pointer + 121;
		      end if;
		      
		      r3c1_next <= r3c2_reg;
		      r3c2_next <= r3c3_reg;

	        state_next <= write1;
        
	      when write1 =>
	        
          --read_write_addr <= write_pointer;
          
	        if(sum < 3 or sum > 4) then
 	          write_data <= '0' & r2c2_reg(2 downto 0);
	        elsif(sum = 3 and r2c2_reg(3) = '0') then
	       		--write_data <= '1' & r2c2_reg(2 downto 0);
	        	write_data <= "1111";
	        else
	          write_data <= r2c2_reg(3 downto 0);
	        end if;

		

	        -- shifts column 2 to column 1
          --r1c1_next <= r1c2_reg;
	        --r2c1_next <= r2c2_reg;
	        --r3c1_next <= r3c2_reg;
	        -- shifts column 3 to column 2
          --r1c2_next <= r1c3_reg;
	        --r2c2_next <= r2c3_reg;
	        --r3c2_next <= r3c3_reg;

	        
	        
	        if(write_pointer = 19200) then
	          frame <= not frame;
	          write_pointer_next <= (others=>'0');
	          state_next <= newline2;
		  x_place_reg <= (others=>'0');
	        else
			
	          write_pointer_next <= write_pointer + 1;
	          state_next <= idle;
	        end if;
	        
	        
	     when rand_init =>
					--we <= '1';
					
					write_pointer_next <= (others=>'0');
					
					r1c1_next <= (others=>'0');
          r2c1_next <= (others=>'0');
          r3c1_next <= (others=>'0');
          r1c2_next <= (others=>'0');
          r2c2_next <= (others=>'0');
          r3c2_next <= (others=>'0');
					r1c3_next <= (others=>'0');
					r2c3_next <= (others=>'0');
					r3c3_next <= (others=>'0');
					
					write_data <= init_data;
					 --write_data <= "111" & switches(7);
					--read_write_addr <= unsigned(init_addr);
	            if init_done='1' then
			    x_place_reg <= (others =>'0');
	              state_next <= newline2;
              else 
                state_next <= rand_init;
            end if;
            
       when nothing =>
            state_next <= nothing;

        end case;
  end process;
  
  process(button2, button3)
    begin
      if(button2 = '1') then
        speed_next <= speed;
      else
        speed_next <= speed_reg;
      end if;
  end process;
  
  sum <= ((unsigned(r1c1_reg(6 downto 3)) + unsigned(r1c2_reg(6 downto 3))) + 
         (unsigned(r1c3_reg(6 downto 3)) + unsigned(r2c1_reg(6 downto 3)))) +
         ((unsigned(r2c3_reg(6 downto 3)) + unsigned(r3c1_reg(6 downto 3))) + 
         (unsigned(r3c2_reg(6 downto 3)) + unsigned(r3c3_reg(6 downto 3))));
      
   --sum <= "0001";   
      
  speed <= "00000000" &
           switches(7) & '0' & switches(6) & '0' & switches(5) & '0' &
           switches(4) & '0' & switches(3) & '0' & switches(2) & '0' &
           switches(1) & '0' & switches(0) & "000";
   
--   sum <= ((unsigned(r1c1_next(6 downto 3)) + unsigned(r1c2_next(6 downto 3))) + 
--         (unsigned(r1c3_next(6 downto 3)) + unsigned(r2c1_next(6 downto 3)))) +
--         ((unsigned(r2c3_next(6 downto 3)) + unsigned(r3c1_next(6 downto 3))) + 
--         (unsigned(r3c2_next(6 downto 3)) + unsigned(r3c3_next(6 downto 3))));
  
read_write_addr <= unsigned(init_addr) when state_next = rand_init else
		     write_pointer  -160 when state_next = newline1 else
                     write_pointer       when state_next = newline2 else
                     write_pointer + 160 when state_next = newline3 else
                     write_pointer - 159 when state_next = read1 else
                     write_pointer + 1   when state_next = read2 else
                     write_pointer + 161 when state_next = read3 else
                     write_pointer;
                     
  
  --we1 <= '1' when (state_reg = write1 or state_reg = rand_init) and frame = '0' else '0';
  --we2 <= '1' when (state_reg = write1 or state_reg = rand_init) and frame = '1' else '0';

  --read_write_value <= read_write_value1 when frame = '1' else
    --                  read_write_value2;
                      
  --read_value <= read_value1 when frame = '1' else
  --                    read_value2;
  
  
  we1 <= '1' when (state_reg = write1 or state_reg = rand_init)  else '0';
  
  read_write_value <= read_write_value1;
  read_value <= read_value1;

  -- used to calcualte address of pixels
  vga_addr <= ((unsigned(pixel_y)/4) * 160) + unsigned(pixel_x)/4;
  
  -- if dead display black, else display color
  --RGB <= "000" when read_value(3) = '0' or blank = '1' else read_value(2 downto 0);
  -- if dead display background color, else display color+background color
  RGB <= "000" when blank = '1'  else 
        std_logic_vector(background_color) when read_value(3) = '0' else
        std_logic_vector(unsigned(read_value(2 downto 0)) + background_color);
        
        

    
  
  --read_write_addr <= unsigned(init_addr) when state_reg=rand_init else read_write_addr2;
  
  ----- Random Init ----
    init_en <= button0;
    change_colors <= button1;
  
  
  comp1 : vgatimer 
  port map(
	  clk => clk,
		pixel_en => pixel_in,
		rst => '0',	
		HS => HS,
		VS => VS,
		pixel_x => pixel_x,
		pixel_y => pixel_y,
		last_column => open,
		last_row => open,
		blank => blank
		);
    
  comp2 : default_screen
  port map(
      clk => clk,
      en => '1',
      we => we1,
      addra => std_logic_vector(read_write_addr),
      addrb => std_logic_vector(vga_addr(14 downto 0)),
      di => write_data,
      doa => read_write_value1,
      dob => read_value1
    );
    
    
    rand:Random_Init
      port map(   init_en => init_en,
            density =>switches(6 downto 0),
            clk => clk,
            reset =>reset,
            
            pixel_data => init_data,
            init_done => init_done,
            pixel_addr => init_addr
        );

ss: seven_seg
          --generic map()
         port map( seg0 => std_logic_vector(sum),
                    seg1 => temp1,
                    seg2 => temp2,
                    seg3 => temp3,
                    DP => "1111",
                   segments => segments,
                   clk => clk,
                   an_out => an_sel,
                   dp_out => dp_out
                   );
        
        temp1 <= "000" & r1c3_reg(3);
        temp2 <= "000" & r1c2_reg(3);
        temp3 <= "000" & r1c1_reg(3);
    
  
end controller;

