-- This is a very basic simulation model of the IS61LV25616AL.
--  
-- version 0.1
--  
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity IS61LV25616AL is
  port (
    Address : in  std_logic_vector(17 downto 0);
    dataIO : inout std_logic_vector(15 downto 0);
    CE_bar : in std_logic;
    OE_bar : in std_logic;
    WE_bar : in std_logic;
    UB_bar : in std_logic;
    LB_bar : in std_logic
    );
end IS61LV25616AL;

architecture behavioral of  IS61LV25616AL is

  -- Memory time constants
  constant twc : TIME 	:= 8.0 ns;
  constant tpwe : TIME  := 6.5 ns;
  constant tpwb : TIME  := 6.5 ns;
  constant tsce : TIME  := 6.5 ns;
  constant tsd : TIME  	:= 5.0 ns;
  constant trc : TIME  	:= 8.0 ns;
  constant thzwe : TIME := 3.5 ns;
  constant tlzwe : TIME := 3.0 ns;
  constant thzoe : TIME := 3.0 ns;
  constant thzb : TIME  := 3.0 ns;
  constant tdoe : TIME  := 5.5 ns;
  constant tba : TIME   := 5.5 ns;
  constant taa : TIME  	:= 8.0 ns;
  constant toha : TIME 	:= 2.5 ns;
  constant taw : TIME  	:= 6.5 ns;
  constant tace : TIME	:= 8.0 ns;  
  constant thzce : TIME	:= 8.0 ns;
  constant pulse_time : TIME := 1 ns;
    
  constant RAM_ADDR_WIDTH: integer := 18;
  constant RAM_BANK_WIDTH: integer := 8;
  type char_ram_type is array (0 to 2**RAM_ADDR_WIDTH-1)
    of std_logic_vector(RAM_BANK_WIDTH-1 downto 0);
  --type ram_addr_type is std_logic_vector(RAM_ADDR_WIDTH-1 downto 0);
  signal mem_array_LB : char_ram_type;
  signal mem_array_UB : char_ram_type;

  signal dataOut_temp, dataIn_temp_delayed :
    std_logic_vector (2*RAM_BANK_WIDTH-1 downto 0);

  signal addr_Write_delayed, addr_Read_delayed :
         std_logic_vector(RAM_ADDR_WIDTH-1 downto 0);
  
  signal CEUB_bar, CELB_bar : std_logic;
  signal Read_UB_cond, Read_LB_cond, Write_UB_cond, Write_LB_cond : std_logic;
  signal CTRL_dataOut_LB, CTRL_dataOut_UB : std_logic;
  signal tace_p,taceLB_p, taceUB_p, taa_p, tdoe_p, tsce_p, tsceUB_p, tsceLB_p, tsd_p, tpwe_p, tbaLB_p, tbaUB_p : std_logic := '0';
  signal taw_p, tlzceUB_p, tlzceLB_p, twc_p : std_logic := '0';

begin

  -- Intermediate signals
  CEUB_bar <= CE_bar or UB_bar;
  CELB_bar <= CE_bar or LB_bar; 
  Read_UB_cond 	<=  (not CEUB_bar) and WE_bar and (not OE_bar) ;
  Read_LB_cond 	<=  (not CELB_bar) and WE_bar and (not OE_bar) ;
  Write_LB_cond <= (not CELB_bar) and (not WE_bar) ;
  Write_UB_cond <= (not CEUB_bar) and (not WE_bar) ;

  -- Data output driver
  dataIO(7 downto 0) <= dataOut_temp(7 downto 0) when CTRL_dataOut_LB = '1'
                        else (others => 'Z');
  dataIO(15 downto 8) <= dataOut_temp(15 downto 8) when CTRL_dataOut_UB = '1'
                        else (others => 'Z');

  -- delayed address & data in
  addr_Read_delayed <= Address after taa;
  addr_Write_delayed <= Address after taw;
  dataIn_temp_delayed <= dataIO after tsd;

  -- control for data ouptput gate
  CTRL_dataOut_UB <= '1' when Read_UB_cond = '1' else '0';

  CTRL_dataOut_LB <= '1' when Read_LB_cond = '1' else '0';

  -- Read process
  process(Read_UB_cond, addr_read_delayed)
  begin
    if (read_UB_cond = '1') then
      dataOut_temp(7 downto 0) <= mem_Array_LB(to_integer(unsigned(addr_read_delayed)));
    end if;    
  end process;
  process(Read_LB_cond, addr_read_delayed)
  begin
    if (read_LB_cond = '1') then
      dataOut_temp(15 downto 8) <= mem_Array_UB(to_integer(unsigned(addr_read_delayed)));
    end if;    
  end process;

  -- Write process
  process(Write_UB_cond,addr_write_delayed)
  begin
    if (Write_UB_cond = '1') then
      mem_array_UB(to_integer(unsigned(addr_write_delayed))) <= dataIn_temp_delayed(15 downto 8);
    end if;
  end process;
  process(Write_LB_cond,addr_write_delayed)
  begin
    if (Write_LB_cond = '1') then
      mem_array_LB(to_integer(unsigned(addr_write_delayed))) <= dataIn_temp_delayed(7 downto 0);
    end if;
  end process;
 
end behavioral;
