--Random Init

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.my320pkg.all;


entity Random_Init is
    port(   init_en : in std_logic;
            density : in std_logic_vector(6 downto 0);
            clk : in std_logic;
            reset : in std_logic;
            
            pixel_data : out std_logic_vector(3 downto 0);
            init_done : out std_logic;
            pixel_addr : out std_logic_vector(14 downto 0)
        );
    
end Random_Init;

architecture rand of Random_Init is
signal random_out, random_out_reg : std_logic_vector(3 downto 0);
signal pixel_addr_reg, pixel_addr_next : unsigned(14 downto 0):= (others=>'0');
signal lfsr_out : std_logic_vector(4 downto 0);
type mc_state_type is (idle, init );
signal cur_state : mc_state_type := idle;
signal next_state : mc_state_type;
begin


    randomizer:lfsr 
   port map(
      clk=>clk, 
      reset => reset,
      q=> lfsr_out
   );

  lookup:Lookup_Table 
   port map(
      clk => clk,
      addr => lfsr_out,
   	switches => density,
      data=>random_out
   );


process(clk)
  begin
    if(clk'event and clk='1') then
      random_out_reg <= random_out;
      cur_state <= next_state;
      --if(pixel_addr_reg =19200) then
       --   pixel_addr_next <= (others=>'0');
        --else pixel_addr_next <= pixel_addr_reg;
		  pixel_addr_reg <= pixel_addr_next;
      
    end if;
end process;


process(cur_state, init_en, pixel_addr_reg)
  begin
    case cur_state is
      
        when idle =>
          if init_en='1' then
            next_state <= init;
          else next_state <= idle;
          end if;
            
        when init=>
            if(pixel_addr_reg =19200) then
              pixel_addr_next <= (others=>'0');
              next_state <= idle;
            else pixel_addr_next <= pixel_addr_reg +1;
              next_state <= init;
            end if;
          
      end case;
    
    end process;

pixel_addr <= std_logic_vector(pixel_addr_next);
pixel_data <= random_out_reg;
init_done <= '1' when pixel_addr_reg =19200 else '0';

end rand;
