library ieee;

use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity SyncMemController is
  
  port(clk: in std_logic;
      addr : in std_logic_vector(17 downto 0);
      data_fs : in std_logic_vector(15 downto 0);
      mem : in std_logic;
      rw : in std_logic;
      
      data_sf: out std_logic_vector(15 downto 0);
      addr_io: out std_logic_vector(17 downto 0);
      ce_io : out std_logic;
      we_io : out std_logic;
      oe_io : out std_logic;
      ub_io : out std_logic;
      lb_io : out std_logic;
      ready : out std_logic;
      data_valid : out std_logic;
      
      data_io : inout std_logic_vector(15 downto 0));
            
end SyncMemController;

architecture sControl of SyncMemController is

signal addr_reg, addr_next : std_logic_vector(17 downto 0);
type mc_state_type is (idle, write1, write2, write3, read1, read2);
signal cur_state : mc_state_type := idle;
signal next_state : mc_state_type;
signal data_m2s_next, data_s2m_next, data_m2s_reg, data_s2m_reg : std_logic_vector(15 downto 0);
signal tri_state_en : std_logic;

begin


-------- Raddr --------
process(clk)
begin
  if(clk'event and clk='1') then  
    addr_reg <= addr_next;
    data_s2m_reg <= data_s2m_next;
    data_m2s_reg <= data_m2s_next;
    cur_state <= next_state;
  end if;
end process;
  
  
  
  
  
  
----- FSM -----
process(cur_state, mem, rw, data_io, addr_reg, data_m2s_reg, data_s2m_reg, data_fs, addr)
  begin
    addr_next <= addr_reg;
    data_m2s_next <= data_m2s_reg;
    data_s2m_next <= data_s2m_reg;
    addr_io <= (others=>'0');
    case cur_state is 
        when idle =>
            if (mem='1')then
                if(rw ='1') then
                  next_state <= read1;

                else 
                    next_state <= write1;
                    data_m2s_next <= data_fs;
              end if;
              addr_next<= addr;
            else next_state <= idle;
              
            end if;
            
        when read1 =>
          data_s2m_next <= data_io;
          next_state <= read2;
          addr_io <= addr_reg;
        
      when read2 =>
          next_state <= idle;
          addr_io <= addr_reg;
          data_sf <= data_s2m_reg;
          
          
      when  write1 =>
          next_state <= write2;
          addr_io <= addr_reg;
          
      when write2 =>
          next_state <= write3;
          addr_io <= addr_reg;
        
      when write3 =>
          next_state <= idle;
          addr_io <= addr_reg;
          
            
    end case;
  end process;
  
  ready <= '1' when cur_state=idle else '0';
  oe_io <= '0' when cur_state = read1 else '1';
  we_io <= '0' when cur_state =write2 else '1';
  data_valid <= '1' when cur_state=read2 else '0';  
  ce_io <= '0';
  ub_io <= '0';
  lb_io<= '0';
  tri_state_en <= '1' when (cur_state=write1 or cur_state=write2 or cur_state=write3) else '0';
  
  
  
  ---- Tri-state ----
  data_io <= data_m2s_reg when tri_state_en='1' else (others=>'Z');

end sControl;
