--=============================
-- LFSR
--=============================
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
entity lfsr is
 port(
      clk, reset: in std_logic;
      q: out std_logic_vector(4 downto 0)
   );
end lfsr;

architecture zero_arch of lfsr is
   signal r_reg, r_next: std_logic_vector(4 downto 0);
   signal fb, zero, fzero: std_logic;
   constant seed: std_logic_vector(4 downto 0):="00000";
begin
   -- register
   process(clk,reset)
   begin
      if (reset='1') then
         r_reg <= seed;
      elsif (clk'event and clk='1') then
         r_reg <= r_next;
      end if;
   end process;
   -- next-state logic
   fb <= r_reg(2) xor r_reg(0);
   zero <= '1' when r_reg(4 downto 1)="0000" else
           '0';
   fzero <= zero xor fb;
   r_next <= fzero & r_reg(3 downto 1);
   -- output logic
   q <= r_reg;

end zero_arch;
