  library ieee;
  use ieee.std_logic_1164.all;
  
  package my320pkg is
    component SevenSeg is
    generic (
      COUNTER_BITS : natural := 15
    );
    port (
      clk : in std_logic;
      seg0 : in std_logic_vector(3 downto 0);
      seg1 : in std_logic_vector(3 downto 0);
      seg2 : in std_logic_vector(3 downto 0);
      seg3 : in std_logic_vector(3 downto 0);
      DP : in std_logic_vector(3 downto 0);
    
      segments : out std_logic_vector(6 downto 0);
      dp_out : out std_logic;
      an_out : out std_logic_vector(3 downto 0)
    );
      end component;
    
    component vgatimer is

	  port(
	  clk : in std_logic;
		pixel_en : in std_logic;
		rst : in std_logic;
		
		HS : out std_logic;
		VS : out std_logic;
		pixel_x : out std_logic_vector(9 downto 0);
		pixel_y : out std_logic_vector(9 downto 0);
		last_column : out std_logic;
		last_row : out std_logic;
		blank : out std_logic

		);
    end component;
    
    
    component tx 
		generic(
      CLK_RATE : natural := 50000000;
      BAUD_RATE : natural := 19200;
      BIT_TIMER_WIDTH : natural := 12
    );
      
    port(
       clk, rst : in std_logic;
       send_character : in std_logic;
       data_in : in std_logic_vector(7 downto 0);
    
       tx_out, tx_busy : out std_logic
    );
	   end component;
	   
	   
	   component Mem_Controller
	     port(
          clk : in std_logic;
          addr : in std_logic_vector(17 downto 0);
          data_fs : in std_logic_vector(15 downto 0);
          mem : in std_logic;
          rw : in std_logic;
    
          data_sf : out std_logic_vector(15 downto 0);
          addr_io : out std_logic_vector(17 downto 0);
          ce_io, we_io, oe_io, ub_io, lb_io : out std_logic;
          ready : out std_logic;
          data_valid : out std_logic;
    
          data_io : inout std_logic_vector(15 downto 0)  
        );
    end component;
    
    component Frame_Buffer
  port(
    clk : in std_logic;
    data : in std_logic_vector(15 downto 0);    
    last_row, last_column : in std_logic;
    blank : in std_logic;
    ready : in std_logic;
    
    write_addr : in std_logic_vector(17 downto 0);
    input_color : in std_logic_vector(2 downto 0);
    clear : in std_logic;
    frame : in std_logic;
    pixel_select : in std_logic_vector(1 downto 0);
    change_pixel : in std_logic;
    pixel_x : in std_logic_vector(9 downto 0);
    
    data_write : out std_logic_vector(15 downto 0);
    addr_out : out std_logic_vector(17 downto 0);
    RGB : out std_logic_vector(2 downto 0);
    rw : out std_logic;
    mem : out std_logic
    
  );
    end component;
    
    
    component IS61LV25616AL is
    port (
      Address : in  std_logic_vector(17 downto 0);
      dataIO : inout std_logic_vector(15 downto 0);
      CE_bar : in std_logic;
      OE_bar : in std_logic;
      WE_bar : in std_logic;
      UB_bar : in std_logic;
      LB_bar : in std_logic
    );
    end component;
    
    
    component lfsr is
   port(
      clk, reset: in std_logic;
      q: out std_logic_vector(4 downto 0)
   );
end component;

    component lfsr_time is
   port(
      clk, reset: in std_logic;
      q: out std_logic_vector(3 downto 0)
   );
end component;

component lfsr_32 is

   port(

      clk, reset: in std_logic;

      q: out std_logic_vector(31 downto 0)

   );

end component;

component lfsr_64 is

   port(

      clk, reset: in std_logic;

      q: out std_logic_vector(63 downto 0)

   );

end component;


component lfsr_1024 is

   port(

      clk, reset: in std_logic;

      q: out std_logic_vector(1023 downto 0)

   );

end component;


component Lookup_Table is
   port(
      clk: in std_logic;
      addr: in std_logic_vector(4 downto 0);
   	switches: in std_logic_vector(6 downto 0);
      data: out std_logic_vector(3 downto 0)
      
   );
end component;

component Random_Init is
    port(   init_en : in std_logic;
            density : in std_logic_vector(6 downto 0);
            clk : in std_logic;
            reset : in std_logic;
            
            pixel_data : out std_logic_vector(3 downto 0);
            init_done : out std_logic;
            pixel_addr : out std_logic_vector(14 downto 0)
        );
    
end component;
    
  end my320pkg;
