library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

----------------------------------------------------------------------
-- seven_seg.vhd
--
-- Design for Lab3
----------------------------------------------------------------------
entity seven_seg is
  
  generic (
    COUNTER_BITS : natural :=15 
    );
  port (
    seg0, seg1, seg2, seg3, DP : in  std_logic_vector(3 downto 0);
    clk: in std_logic;
    segments : out std_logic_vector(6 downto 0);
    an_out : out std_logic_vector(3 downto 0);
    dp_out : out std_logic
  );

end seven_seg;

architecture controller of seven_seg is
signal count : unsigned(COUNTER_BITS-1 downto 0) := (others => '0'); 
signal segs : std_logic_vector(3 downto 0);
signal section: std_logic_vector(1 downto 0);
begin


 -----------------------------------------------------------------------------
  -- Counter operating at 50 MHz
  -----------------------------------------------------------------------------
  process (clk)
  begin
    if clk='1' and clk'event then
      count <= count+1;
    end if;
    
  end process;   
  
  section <= count(COUNTER_BITS-1) &  count(COUNTER_BITS-2);
  
 -----------------------------------------------------------------------------
  -- 2:4 decoder
  -----------------------------------------------------------------------------
  with section select
  an_out <=     "1110" when "00",
                "1101" when "01",
                "1011" when "10",
                "0111" when others;
                
                
                
  -----------------------------------------------------------------------------
  -- Decimal Points
  -----------------------------------------------------------------------------
  with section select
  dp_out <=     DP(0) when "00",
                DP(1) when "01",
                DP(2) when "10",
                DP(3) when others;
                
  -----------------------------------------------------------------------------
  -- Seven Segment Mux
  -----------------------------------------------------------------------------
  with section select
  segs <=       seg0 when "00",
                seg1 when "01",
                seg2 when "10",
                seg3 when others;
                
  -----------------------------------------------------------------------------
  -- Seven Segment Decoder
  -----------------------------------------------------------------------------
  with segs select
  segments <=   "0000001"   when "0000",
                "1001111"   when "0001",
                "0010010"   when "0010",
                "0000110"   when "0011",
                "1001100"   when "0100",
                "0100100"   when "0101",
                "0100000"   when "0110",
                "0001111"   when "0111",
                "0000000"   when "1000",
                "0000100"   when "1001",
                "0001000"   when "1010",
                "1100000"   when "1011",
                "0110001"   when "1100",
                "1000010"   when "1101",
                "0110000"   when "1110",
                "0111000"   when others;
                

end controller;



 