-- Listing 13.1
-- ROM with synchonous read (inferring Block RAM)
-- character ROM
--   - 8-by-16 (8-by-2^4) font
--   - 128 (2^7) characters
--   - ROM size: 512-by-8 (2^11-by-8) bits
--               16K bits: 1 BRAM

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;
entity default_screen is
   port(
      clk: in std_logic;
      --addr: in std_logic_vector(10 downto 0);
      --data: out std_logic_vector(7 downto 0)
	en : in std_logic;
	we : in std_logic;
	addra : in std_logic_vector(14 downto 0);
	addrb : in std_logic_vector(14 downto 0);
	di : in std_logic_vector(3 downto 0);
	doa : out std_logic_vector(3 downto 0);
	dob : out std_logic_vector(3 downto 0)
   	
   );
end default_screen;

architecture arch of default_screen is
   constant ADDR_WIDTH: integer:=15;
   constant DATA_WIDTH: integer:=4;
   --type ram_type is array (31 downto 0) of std_logic_vector (3 downto 0);
  --signal RAM : ram_type;
  signal read_addra : std_logic_vector(ADDR_WIDTH-1 downto 0);
  signal read_addrb : std_logic_vector(ADDR_WIDTH-1 downto 0);
   --signal addr_reg: std_logic_vector(ADDR_WIDTH-1 downto 0);
   type rom_type is array (0 to 2**ADDR_WIDTH-1)
        of std_logic_vector(DATA_WIDTH-1 downto 0);
   -- ROM definition
   signal ROM: rom_type:=(   -- 2^11-by-8
    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f



    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f



    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f


    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f


    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f


    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f


    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f


    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f


    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f


    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f


    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f



    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f



    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f



    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f



    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f



    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f
"0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f



    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f



    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f


    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f


    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f


    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f


    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f


    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f


    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f


    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f


    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f



    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f



    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f



    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f



    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000",  -- f



    -- FIRST ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x01
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2  ******
   "1000", -- 3 *      *
   "1010", -- 4 * *  * *
   "1000", -- 5 *      *
   "1000", -- 6 *      *
   "1011", -- 7 * **** *
   "1001", -- 8 *  **  *
   "1000", -- 9 *      *
   "1000", -- a *      *
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x02
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ******
   "1111", -- 3 ********
   "1101", -- 4 ** ** **
   "1111", -- 5 ********
   "1111", -- 6 ********
   "1100", -- 7 **    **
   "1110", -- 8 ***  ***
   "1111", -- 9 ********
   "1111", -- a ********
   "0111", -- b  ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x03
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "1111", -- 6 *******
   "1111", -- 7 *******
   "1111", -- 8 *******
   "0111", -- 9  *****
   "0011", -- a   ***
   "0001", -- b    *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x04
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0111", -- 6  *****
   "1111", -- 7 *******
   "0111", -- 8  *****
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x05
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0011", -- 5   ****
   "1110", -- 6 ***  ***
   "1110", -- 7 ***  ***
   "1110", -- 8 ***  ***
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x06
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0011", -- 4   ****
   "0111", -- 5  ******
   "1111", -- 6 ********
   "1111", -- 7 ********
   "0111", -- 8  ******
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x07
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0001", -- 6    **
   "0011", -- 7   ****
   "0011", -- 8   ****
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x08
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1110", -- 6 ***  ***
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1110", -- 9 ***  ***
   "1111", -- a ********
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x09
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   ****
   "0110", -- 6  **  **
   "0100", -- 7  *    *
   "0100", -- 8  *    *
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- SECOND ROW STARTS HERE
   "1111", -- 0 ********
   "1111", -- 1 ********
   "1111", -- 2 ********
   "1111", -- 3 ********
   "1111", -- 4 ********
   "1100", -- 5 **    **
   "1001", -- 6 *  **  *
   "1011", -- 7 * **** *
   "1011", -- 8 * **** *
   "1001", -- 9 *  **  *
   "1100", -- a **    **
   "1111", -- b ********
   "1111", -- c ********
   "1111", -- d ********
   "1111", -- e ********
   "1111", -- f ********
   -- code x0b
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     ***
   "0001", -- 4    ** *
   "0011", -- 5   **  *
   "0111", -- 6  ****
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0111", -- 9  ******
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ******
   "0011", -- 3   **  **
   "0011", -- 4   ******
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0111", -- 9  ***
   "1111", -- a ****
   "1110", -- b ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "0110", -- 3  **   **
   "0111", -- 4  *******
   "0110", -- 5  **   **
   "0110", -- 6  **   **
   "0110", -- 7  **   **
   "0110", -- 8  **   **
   "0110", -- 9  **  ***
   "1110", -- a ***  ***
   "1110", -- b ***  **
   "1100", -- c **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x0f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0001", -- 3    **
   "0001", -- 4    **
   "1101", -- 5 ** ** **
   "0011", -- 6   ****
   "1110", -- 7 ***  ***
   "0011", -- 8   ****
   "1101", -- 9 ** ** **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x10
   "0000", -- 0
   "1000", -- 1 *
   "1100", -- 2 **
   "1110", -- 3 ***
   "1111", -- 4 ****
   "1111", -- 5 *****
   "1111", -- 6 *******
   "1111", -- 7 *****
   "1111", -- 8 ****
   "1110", -- 9 ***
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x11
   "0000", -- 0
   "0000", -- 1       *
   "0000", -- 2      **
   "0000", -- 3     ***
   "0001", -- 4    ****
   "0011", -- 5   *****
   "1111", -- 6 *******
   "0011", -- 7   *****
   "0001", -- 8    ****
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x12
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- THIRD ROW STARTS HERE
   "0000", -- 0
   "0000", -- 1
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0000", -- 9
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x14
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *******
   "1101", -- 3 ** ** **
   "1101", -- 4 ** ** **
   "1101", -- 5 ** ** **
   "0111", -- 6  **** **
   "0001", -- 7    ** **
   "0001", -- 8    ** **
   "0001", -- 9    ** **
   "0001", -- a    ** **
   "0001", -- b    ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x15
   "0000", -- 0
   "0111", -- 1  *****
   "1100", -- 2 **   **
   "0110", -- 3  **
   "0011", -- 4   ***
   "0110", -- 5  ** **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "0110", -- 8  ** **
   "0011", -- 9   ***
   "0000", -- a     **
   "1100", -- b **   **
   "0111", -- c  *****
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x16
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "1111", -- 8 *******
   "1111", -- 9 *******
   "1111", -- a *******
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x17
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0111", -- 8  ******
   "0011", -- 9   ****
   "0001", -- a    **
   "0111", -- b  ******
   "0011", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x18
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0111", -- 4  ******
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x19
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0111", -- 9  ******
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0000", -- 6     **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0011", -- 5   **
   "0110", -- 6  **
   "1111", -- 7 *******
   "0110", -- 8  **
   "0011", -- 9   **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1111", -- 9 *******
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0010", -- 5   *  *
   "0110", -- 6  **  **
   "1111", -- 7 ********
   "0110", -- 8  **  **
   "0010", -- 9   *  *
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0011", -- 6   ***
   "0111", -- 7  *****
   "0111", -- 8  *****
   "1111", -- 9 *******
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x1f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1111", -- 4 *******
   "1111", -- 5 *******
   "0111", -- 6  *****
   "0111", -- 7  *****
   "0011", -- 8   ***
   "0011", -- 9   ***
   "0001", -- a    *
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x20
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x21
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0011", -- 3   ****
   "0011", -- 4   ****
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x22
   "0000", -- 0
   "0110", -- 1  **  **
   "0110", -- 2  **  **
   "0110", -- 3  **  **
   "0010", -- 4   *  *
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x23
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "1111", -- 5 *******
   "0110", -- 6  ** **
   "0110", -- 7  ** **
   "0110", -- 8  ** **
   "1111", -- 9 *******
   "0110", -- a  ** **
   "0110", -- b  ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x24
   "0001", -- 0     **
   "0001", -- 1     **
   "0111", -- 2   *****
   "1100", -- 3  **   **
   "1100", -- 4  **    *
   "1100", -- 5  **
   "0111", -- 6   *****
   "0000", -- 7       **
   "0000", -- 8       **
   "1000", -- 9  *    **
   "1100", -- a  **   **
   "0111", -- b   *****
   "0001", -- c     **
   "0001", -- d     **
   "0000", -- e
   "0000", -- f
   -- code x25
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "1100", -- 4 **    *
   "1100", -- 5 **   **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1000", -- b *    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x26
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  ** **
   "0011", -- 5   ***
   "0111", -- 6  *** **
   "1101", -- 7 ** ***
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x27
   "0000", -- 0
   "0011", -- 1   **
   "0011", -- 2   **
   "0011", -- 3   **
   "0110", -- 4  **
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x28
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0001", -- a    **
   "0000", -- b     **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x29
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   **
   "0001", -- 3    **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "1111", -- 7 ********
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0001", -- 5    **
   "0001", -- 6    **
   "0111", -- 7  ******
   "0001", -- 8    **
   "0001", -- 9    **
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0011", -- c   **
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0111", -- 7  ******
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x2f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4       *
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **
   "1000", -- b *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x30
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **  ***
   "1101", -- 6 ** ****
   "1111", -- 7 **** **
   "1110", -- 8 ***  **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x31
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2
   "0011", -- 3
   "0111", -- 4    **
   "0001", -- 5   ***
   "0001", -- 6  ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b    **
   "0000", -- c    **
   "0000", -- d  ******
   "0000", -- e
   "0000", -- f
   -- code x32
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x33
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0011", -- 6   ****
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x34
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     **
   "0001", -- 3    ***
   "0011", -- 4   ****
   "0110", -- 5  ** **
   "1100", -- 6 **  **
   "1111", -- 7 *******
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0001", -- b    ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x35
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x36
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  **
   "1100", -- 4 **
   "1100", -- 5 **
   "1111", -- 6 ******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x37
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "1100", -- 3 **   **
   "0000", -- 4      **
   "0000", -- 5      **
   "0000", -- 6     **
   "0001", -- 7    **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x38
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  *****
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x39
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "0111", -- 6  ******
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a     **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3      **
   "0000", -- 4     **
   "0001", -- 5    **
   "0011", -- 6   **
   "0110", -- 7  **
   "0011", -- 8   **
   "0001", -- 9    **
   "0000", -- a     **
   "0000", -- b      **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ******
   "0000", -- 6
   "0000", -- 7
   "0111", -- 8  ******
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0110", -- 3  **
   "0011", -- 4   **
   "0001", -- 5    **
   "0000", -- 6     **
   "0000", -- 7      **
   "0000", -- 8     **
   "0001", -- 9    **
   "0011", -- a   **
   "0110", -- b  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x3f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0000", -- 5     **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0000", -- 9
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x40
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1101", -- 6 ** ****
   "1101", -- 7 ** ****
   "1101", -- 8 ** ****
   "1101", -- 9 ** ***
   "1100", -- a **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x41
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   ***
   "0110", -- 4  ** **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x42
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1111", -- b ******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x43
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **    *
   "0110", -- a  **  **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x44
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *****
   "0110", -- 3  ** **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  ** **
   "1111", -- b *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x45
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x46
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 *******
   "0110", -- 3  **  **
   "0110", -- 4  **   *
   "0110", -- 5  ** *
   "0111", -- 6  ****
   "0110", -- 7  ** *
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x47
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0110", -- 3  **  **
   "1100", -- 4 **    *
   "1100", -- 5 **
   "1100", -- 6 **
   "1101", -- 7 ** ****
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "0110", -- a  **  **
   "0011", -- b   *** *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x48
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1111", -- 6 *******
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x49
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4a
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***  **
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  ** **
   "0111", -- 6  ****
   "0111", -- 7  ****
   "0110", -- 8  ** **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4c
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ****
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **
   "0110", -- 6  **
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **   *
   "0110", -- a  **  **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4d
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1110", -- 3 ***  ***
   "1111", -- 4 ********
   "1111", -- 5 ********
   "1101", -- 6 ** ** **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "1100", -- 9 **    **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4e
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1110", -- 3 ***  **
   "1111", -- 4 **** **
   "1111", -- 5 *******
   "1101", -- 6 ** ****
   "1100", -- 7 **  ***
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "1100", -- b **   **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x4f
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x50
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x510
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1101", -- 9 ** * **
   "1101", -- a ** ****
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     ***
   "0000", -- e
   "0000", -- f
   -- code x52
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ******
   "0110", -- 3  **  **
   "0110", -- 4  **  **
   "0110", -- 5  **  **
   "0111", -- 6  *****
   "0110", -- 7  ** **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x53
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *****
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "0110", -- 5  **
   "0011", -- 6   ***
   "0000", -- 7     **
   "0000", -- 8      **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x54
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1101", -- 3 ** ** **
   "1001", -- 4 *  **  *
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x55
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **   **
   "1100", -- 3 **   **
   "1100", -- 4 **   **
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x56
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x57
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1111", -- 9 ********
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f

   -- code x58
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "0110", -- 4  **  **
   "0011", -- 5   ****
   "0001", -- 6    **
   "0001", -- 7    **
   "0011", -- 8   ****
   "0110", -- 9  **  **
   "1100", -- a **    **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x59
   "0000", -- 0
   "0000", -- 1
   "1100", -- 2 **    **
   "1100", -- 3 **    **
   "1100", -- 4 **    **
   "0110", -- 5  **  **
   "0011", -- 6   ****
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5a
   "0000", -- 0
   "0000", -- 1
   "1111", -- 2 ********
   "1100", -- 3 **    **
   "1000", -- 4 *    **
   "0000", -- 5     **
   "0001", -- 6    **
   "0011", -- 7   **
   "0110", -- 8  **
   "1100", -- 9 **     *
   "1100", -- a **    **
   "1111", -- b ********
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5b
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0011", -- 3   **
   "0011", -- 4   **
   "0011", -- 5   **
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5c
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "1000", -- 3 *
   "1100", -- 4 **
   "1110", -- 5 ***
   "0111", -- 6  ***
   "0011", -- 7   ***
   "0001", -- 8    ***
   "0000", -- 9     ***
   "0000", -- a      **
   "0000", -- b       *
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5d
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ****
   "0000", -- 3     **
   "0000", -- 4     **
   "0000", -- 5     **
   "0000", -- 6     **
   "0000", -- 7     **
   "0000", -- 8     **
   "0000", -- 9     **
   "0000", -- a     **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5e
   "0001", -- 0    *
   "0011", -- 1   ***
   "0110", -- 2  ** **
   "1100", -- 3 **   **
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x5f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "1111", -- d ********
   "0000", -- e
   "0000", -- f
   -- code x60
   "0011", -- 0   **
   "0011", -- 1   **
   "0001", -- 2    **
   "0000", -- 3
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x61
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  ****
   "0000", -- 6     **
   "0111", -- 7  *****
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x62
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2  ***
   "0110", -- 3   **
   "0110", -- 4   **
   "0111", -- 5   ****
   "0110", -- 6   ** **
   "0110", -- 7   **  **
   "0110", -- 8   **  **
   "0110", -- 9   **  **
   "0110", -- a   **  **
   "0111", -- b   *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x63
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x64
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    ***
   "0000", -- 3     **
   "0000", -- 4     **
   "0011", -- 5   ****
   "0110", -- 6  ** **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x65
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1111", -- 7 *******
   "1100", -- 8 **
   "1100", -- 9 **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x66
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0110", -- 3  ** **
   "0110", -- 4  **  *
   "0110", -- 5  **
   "1111", -- 6 ****
   "0110", -- 7  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x67
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "1100", -- d **  **
   "0111", -- e  ****
   "0000", -- f
   -- code x68
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  ** **
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x69
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0000", -- 4
   "0011", -- 5   ***
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2      **
   "0000", -- 3      **
   "0000", -- 4
   "0000", -- 5     ***
   "0000", -- 6      **
   "0000", -- 7      **
   "0000", -- 8      **
   "0000", -- 9      **
   "0000", -- a      **
   "0000", -- b      **
   "0110", -- c  **  **
   "0110", -- d  **  **
   "0011", -- e   ****
   "0000", -- f
   -- code x6b
   "0000", -- 0
   "0000", -- 1
   "1110", -- 2 ***
   "0110", -- 3  **
   "0110", -- 4  **
   "0110", -- 5  **  **
   "0110", -- 6  ** **
   "0111", -- 7  ****
   "0111", -- 8  ****
   "0110", -- 9  ** **
   "0110", -- a  **  **
   "1110", -- b ***  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6c
   "0000", -- 0
   "0000", -- 1
   "0011", -- 2   ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0001", -- 6    **
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0011", -- b   ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6d
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1110", -- 5 ***  **
   "1111", -- 6 ********
   "1101", -- 7 ** ** **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1101", -- a ** ** **
   "1101", -- b ** ** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6e
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x6f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x70
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0110", -- 6  **  **
   "0110", -- 7  **  **
   "0110", -- 8  **  **
   "0110", -- 9  **  **
   "0110", -- a  **  **
   "0111", -- b  *****
   "0110", -- c  **
   "0110", -- d  **
   "1111", -- e ****
   "0000", -- f
   -- code x71
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *** **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *****
   "0000", -- c     **
   "0000", -- d     **
   "0001", -- e    ****
   "0000", -- f
   -- code x72
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1101", -- 5 ** ***
   "0111", -- 6  *** **
   "0110", -- 7  **  **
   "0110", -- 8  **
   "0110", -- 9  **
   "0110", -- a  **
   "1111", -- b ****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x73
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "0111", -- 5  *****
   "1100", -- 6 **   **
   "0110", -- 7  **
   "0011", -- 8   ***
   "0000", -- 9     **
   "1100", -- a **   **
   "0111", -- b  *****
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x74
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    *
   "0011", -- 3   **
   "0011", -- 4   **
   "1111", -- 5 ******
   "0011", -- 6   **
   "0011", -- 7   **
   "0011", -- 8   **
   "0011", -- 9   **
   "0011", -- a   ** **
   "0001", -- b    ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x75
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **  **
   "1100", -- 6 **  **
   "1100", -- 7 **  **
   "1100", -- 8 **  **
   "1100", -- 9 **  **
   "1100", -- a **  **
   "0111", -- b  *** **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x76
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1100", -- 8 **    **
   "0110", -- 9  **  **
   "0011", -- a   ****
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x77
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "1100", -- 6 **    **
   "1100", -- 7 **    **
   "1101", -- 8 ** ** **
   "1101", -- 9 ** ** **
   "1111", -- a ********
   "0110", -- b  **  **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x78
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **    **
   "0110", -- 6  **  **
   "0011", -- 7   ****
   "0001", -- 8    **
   "0011", -- 9   ****
   "0110", -- a  **  **
   "1100", -- b **    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x79
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1100", -- 5 **   **
   "1100", -- 6 **   **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1100", -- a **   **
   "0111", -- b  ******
   "0000", -- c      **
   "0000", -- d     **
   "1111", -- e *****
   "0000", -- f
   -- code x7a
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0000", -- 4
   "1111", -- 5 *******
   "1100", -- 6 **  **
   "0001", -- 7    **
   "0011", -- 8   **
   "0110", -- 9  **
   "1100", -- a **   **
   "1111", -- b *******
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7b
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2     ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0111", -- 6  ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0000", -- b     ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7c
   "0000", -- 0
   "0000", -- 1
   "0001", -- 2    **
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0001", -- b    **
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7d
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  ***
   "0001", -- 3    **
   "0001", -- 4    **
   "0001", -- 5    **
   "0000", -- 6     ***
   "0001", -- 7    **
   "0001", -- 8    **
   "0001", -- 9    **
   "0001", -- a    **
   "0111", -- b  ***
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7e
   "0000", -- 0
   "0000", -- 1
   "0111", -- 2  *** **
   "1101", -- 3 ** ***
   "0000", -- 4
   "0000", -- 5
   "0000", -- 6
   "0000", -- 7
   "0000", -- 8
   "0000", -- 9
   "0000", -- a
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000", -- f
   -- code x7f
   "0000", -- 0
   "0000", -- 1
   "0000", -- 2
   "0000", -- 3
   "0001", -- 4    *
   "0011", -- 5   ***
   "0110", -- 6  ** **
   "1100", -- 7 **   **
   "1100", -- 8 **   **
   "1100", -- 9 **   **
   "1111", -- a *******
   "0000", -- b
   "0000", -- c
   "0000", -- d
   "0000", -- e
   "0000"  -- f
   );
begin
   -- addr register to infer block RAM
   process (clk)
   begin
      if (clk'event and clk = '1') then
        --addr_reg <= addr;
        if (en = '1') then
          if (we = '1') then
            ROM(conv_integer(addra)) <= di;
          end if;
          read_addra <= addra;
          read_addrb <= addrb;
        end if;
    end if;
   end process;
   --data <= ROM(to_integer(unsigned(addr_reg)));
   doa <= ROM(conv_integer(read_addra));
  dob <= ROM(conv_integer(read_addrb));
end arch;

